/* Langton's Ant Simulation
   Author: Aaron Weeden, Shodor, 2015
 */

/* Model parameters */
var STEPS_PER_SIM      = 200;
var PX_PER_CELL_X      = 25;
var PX_PER_CELL_Y      = 25;
var CELLS_PER_GRID_X   = 11;
var CELLS_PER_GRID_Y   = 11;
var MILLIS_PER_TIMEOUT = 1;

/* Global variables */
var Ant   = {};
var Cells = [];
var State = 'turn';
var Step  = 0;

/* Functions */

function moveAnt() {
  'use strict';

  Ant.style.left = (PX_PER_CELL_X * Ant.cellX) + 'px';
  Ant.style.top  = (PX_PER_CELL_Y * Ant.cellY) + 'px';
}

function turnAnt() {
  'use strict';

  Ant.style.transform =
    Ant.style.webkitTransform =
    Ant.style.msTransform =
    'rotate(' + Ant.angle + 'deg)';
}

function move() {
  'use strict';
  switch (Ant.angle) {
    case 0:
      /* up */
      Ant.cellY--;
      break;
    case -90:
      /* left */
      Ant.cellX--;
      break;
    case 90:
      /* down */
      Ant.cellX++;
      break;
    case 180:
      /* right */
      Ant.cellY++;
      break;
  }
  moveAnt();
}

function turn() {
  'use strict';

  if (Cells[Ant.cellX][Ant.cellY].style.backgroundColor === 'black') {
    Ant.angle += 360;
  } else {
    Ant.angle += 180;
  }

  Ant.angle = (Ant.angle % 360) - 90;

  turnAnt();
}

function updateCell() {
  'use strict';
  var c;

  /* Swap cell color */
  Cells[Ant.cellX][Ant.cellY].style.backgroundColor =
    (Cells[Ant.cellX][Ant.cellY].style.backgroundColor === 'black') ?
    'white' : 'black';
}

function advanceSim() {
  'use strict';

  switch (State) {
    case 'turn':
      turn();
      State = 'updateCell';
      break;
    case 'updateCell':
      updateCell();
      State = 'move';
      break;
    case 'move':
      move();
      State = 'turn';
      Step++;
      break;
  }
  if (Step < STEPS_PER_SIM) {
    setTimeout(function () {
      advanceSim();
    }, MILLIS_PER_TIMEOUT);
  }
}

window.onload = function () {
  'use strict';
  var grid = document.getElementById('grid'),
    x,
    y,
    cell;

  /* Bound the grid */
  grid.style.width  = (PX_PER_CELL_X * CELLS_PER_GRID_X) + 'px';
  grid.style.height = (PX_PER_CELL_Y * CELLS_PER_GRID_Y) + 'px';

  /* Create cells */
  for (x = 0; x < CELLS_PER_GRID_X; x++) {
    Cells.push([]);
    for (y = 0; y < CELLS_PER_GRID_Y; y++) {
      Cells[x][y] = document.createElement('div');
      Cells[x][y].className = 'cell';
      Cells[x][y].style.left   = x * PX_PER_CELL_X + 'px';
      Cells[x][y].style.top    = y * PX_PER_CELL_Y + 'px';
      Cells[x][y].style.width  = PX_PER_CELL_X + 'px';
      Cells[x][y].style.height = PX_PER_CELL_Y + 'px';
      grid.appendChild(Cells[x][y]);
    }
  }

  Ant = document.getElementById('ant');

  /* Bound the Ant */
  Ant.width  = PX_PER_CELL_X;
  Ant.height = PX_PER_CELL_Y;

  /* Center the Ant */
  Ant.cellX = Math.floor(CELLS_PER_GRID_X / 2);
  Ant.cellY = Math.floor(CELLS_PER_GRID_Y / 2);
  moveAnt();

  /* Give the Ant direction */
  Ant.angle = -90;
  turnAnt();

  setTimeout(function () {
    advanceSim();
  }, MILLIS_PER_TIMEOUT);
};
